;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; REST API bindings for meta.sr.ht
;;;
;;; Copyright (c) 2019, Evan Hanson
;;;
;;; See LICENSE for details.
;;;

(declare (module (topham meta))
         (export user audit-log ssh-key ssh-keys pgp-key pgp-keys))

(import (chicken format)
        (chicken keyword)
        (chicken type)
        (topham))

(define-inline (make-crud method path #!optional (body '()))
  `((#:service "meta" #:path ,path #:method ,method) . ,body))

(define-inline (make-ssh-key #!key ssh-key)
  `((ssh-key . ,ssh-key)))

(define-inline (make-pgp-key #!key pgp-key)
  `((pgp-key . ,pgp-key)))

;;
;; https://man.sr.ht/meta.sr.ht/user-api.md#get-apiuserprofile
;;
(: user (-> (list-of pair)))
(define (user #!rest args)
  (if (null? args)
      (make-crud 'GET "/api/user/profile")
      (make-crud 'PUT "/api/user/profile" (keyword-arguments->alist args))))

;;
;; https://man.sr.ht/meta.sr.ht/user-api.md#get-apiuseraudit-log
;;
(: audit-log (-> (list-of pair)))
(define (audit-log)
  (make-crud 'GET "/api/user/audit-log"))

;;
;; https://man.sr.ht/meta.sr.ht/user-api.md#get-apiuserssh-keys
;;
(: ssh-keys (-> (list-of pair)))
(define (ssh-keys)
  (make-crud 'GET "/api/user/ssh-keys"))

;;
;; https://man.sr.ht/meta.sr.ht/user-api.md#get-apiuserssh-keysid
;; https://man.sr.ht/meta.sr.ht/user-api.md#post-apiuserssh-keys
;;
(: ssh-key (#!optional any #!rest any -> (list-of pair)))
(define (ssh-key #!optional id #!rest details)
  (cond
    ((integer? id)
     (make-crud 'GET (format "/api/user/ssh-keys/~A" id)))
    ((get-keyword #:ssh-key (cons id details))
     (make-crud 'POST "/api/user/ssh-keys" (apply make-ssh-key id details)))
    (else
     (signal-condition
      '(topham)
      '(arity)
      '(exn location ssh-key message "ssh-key id or #:ssh-key must be given")))))

;;
;; https://man.sr.ht/meta.sr.ht/user-api.md#get-apiuserpgp-keys
;;
(: pgp-keys (-> (list-of pair)))
(define (pgp-keys)
  (make-crud 'GET "/api/user/pgp-keys"))

;;
;; https://man.sr.ht/meta.sr.ht/user-api.md#get-apiuserpgp-keysid
;; https://man.sr.ht/meta.sr.ht/user-api.md#post-apiuserpgp-keys
;;
(: pgp-key (#!optional any #!rest any -> (list-of pair)))
(define (pgp-key #!optional id #!rest details)
  (cond
    ((integer? id)
     (make-crud 'GET (format "/api/user/pgp-keys/~A" id)))
    ((get-keyword #:pgp-key (cons id details))
     (make-crud 'POST "/api/user/pgp-keys" (apply make-pgp-key id details)))
    (else
     (signal-condition
      '(topham)
      '(arity)
      '(exn location pgp-key message "pgp-key id or #:pgp-key must be given")))))
